asso_prof = {
    'VBu': 'Valérie Burlet',
    'SDe': 'Sylvie Demouliere',
    'TDu': 'Thierry Dumartin',
    'AGa': 'Alain Gaugue',
    'BHe': 'Bernard Helly',
    'JKh': 'Jamal Khamlichi',
    'JMa': 'Julien Maitre',
    'MMe': 'Michel Menard',
    'SMe': 'Sebastien Mesure',
    'MSl': 'Magali Slingue',
    'DTh': 'Didier Thibaut',
    'CSo': 'Claire Soret',
    'TBe': 'Tess Brehe',
    'CLi': 'Christophe Liebe',
    'ABo': 'Anthony Bourmaud',
    'KFr': 'Karine Fraval',
    'DEM': 'Sylvie Demouliere',
    'Aba': 'Arthur BAUDET',
    'FDe': 'Frederic DEMIER',
    'kCh': 'Karine CHARPIAT',
    'GCh': 'Gérard CHALHOUB',
    'AMaC': 'Ana-Maria Constantin'
}
